/*------------------------------------------------------------------.
| Return the path to the skeleton which locaction might be given in |
| ENVVAR, otherwise return SKELETON_NAME.                           |
`------------------------------------------------------------------*/
/*Correct by Shura (debil_urod@ngs.ru) */
const char *
skeleton_find (const char *envvar, const char *skeleton_name)
{
  const char *res = getenv (envvar);

#if defined(MSDOS) || defined(_WIN32)
  if(!res) {
      const char *cp;
      const char *skel_name = strrchr(skeleton_name, '/');
      if (!skel_name)
        skel_name = strrchr(skeleton_name, '\\');
      if (!skel_name)
        skel_name = skeleton_name;
      else
        ++skel_name;

      /* Try obtain INIT directory. */
      cp = getenv ("INIT");
      if (cp) {
        res = XMALLOC (char, strlen (cp) + strlen (skel_name) + 2);
        sprintf (res, "%s%c%s", cp, '\\', skel_name);
      } else if (access(skel_name, 4) == 0) /* Find file in current dir */
        res = skel_name;
      else { 
        /* File doesn't exist in current directory; try in program location */
        extern char *program_name;
        cp = strrchr(program_name, '\\');
        if (!cp) { /* program_name don't contain program location */
#ifndef MSDOS /* _WIN32 */ /* Call WinApi for obtain it */
          size_t len = 128, res;
          char* mod_name = 0;
          do {
            extern size_t __stdcall GetModuleFileNameA(void*, char*, size_t);
            mod_name = XREALLOC(mod_name, char, len <<=1);
            res = GetModuleFileNameA(NULL, mod_name, len);
          } while (res == len && mod_name[len - 1] != 0);
          cp = strrchr(mod_name, '\\') + 1;
          len = cp - mod_name;
          mod_name = XREALLOC(mod_name, char, len + strlen (skel_name) + 1);
          strcpy(mod_name + len, skel_name);
          return mod_name;
#else  /* MSDOS */
          /*
          I don't known how it can be made by common way for dos-compilers
          without undocumented dos call. ;-(
          */
          return skeleton_name;
#endif /* MSDOS */
        } else 
          ++cp;
        res = XMALLOC (char, cp - program_name + strlen (skel_name) + 1);
        strncpy(res, program_name, cp - program_name);
        strcpy(res + (cp - program_name), skel_name);
      }
    }
#endif /* defined(MSDOS) || defined(_WIN32) */
  if (!res)
    res = skeleton_name;

  return res;
}

/* Gets the extension index in FILENAME. Returns 0 if fails to
   find an extension.  */
/*Correct by Shura (debil_urod@ngs.ru) */
static int
get_extension_index (const char *filename)
{
  int len;

  len = strlen (filename);

  if (filename[len-- - 1] == '.')
    return (0);

  while ((len > 0) && (filename[len - 1] != '.'))
#if defined(MSDOS) || defined(_WIN32)
    switch (filename[--len])
      case '\\': case '/':
        return (0);
#else /* Common Unix */
    if (filename[len - 1] == '/')
      return (0);
    else
      len--;
#endif /* defined(MSDOS) || defined(_WIN32) */

  return (len - 1);
}
